
import statsmodels.api as sm
import numpy as np



axis = 1

def abs_energy(data):
    """Calculates Absolute Energy

    Parameters
    ----------

    data : Matrix or DataFrame
        Data to perform operation on

    Returns
    -------
    Array
        1-dim array
    """
    return np.sum(data**2, axis=axis)

def area_under_curve(data):
    """Calculates AUC

    Parameters
    ----------

    data : Matrix or DataFrame
        Data to perform operation on

    Returns
    -------
    Array
        1-dim array
    """
    return np.trapz(data, axis=axis)

def mean_abs_diff(data):
    """Calculates Mean Absolute Difference

    Parameters
    ----------

    data : Matrix or DataFrame
        Data to perform operation on

    Returns
    -------
    Array
        1-dim array
    """
    return(np.mean(np.abs(np.diff(data)), axis=axis))

def moment(data):
    """Calculates Moment

    Parameters
    ----------

    data : Matrix or DataFrame
        Data to perform operation on

    Returns
    -------
    Array
        1-dim array
    """
    return moment(data, axis=axis)

# Calculate autocorrelation function - default 2
def autocorrelate(data, lag=2):
    """Calculates AutoCorrelation

    Parameters
    ----------

    data : Matrix or DataFrame
        Data to perform operation on
    lag : int
        AutoCorrelation Lag

    Returns
    -------
    Array
        1-dim array
    """
    def _autoc(row, lag):
        return(sm.tsa.acf(row, nlags=lag, fft=False))

    return(np.apply_along_axis(_autoc, data, lag=lag)[:, lag])

## Calculate Zero Crossing rate
def zero_crossing(data):
    """Calculates Zero Crossing Rate

    Parameters
    ----------

    data : Matrix or DataFrame
        Data to perform operation on

    Returns
    -------
    Array
        1-dim array
    """
    def _cross(row):
        return(np.where(np.diff(row > 0)))

    return(np.apply_along_axis(_cross, 1, data))



import pandas as pd
import numpy as np
from datetime import datetime
from pandas import concat



## Description takes in 1-dim data, T+, window size
### data - 1 dim time series
### f_idx - T+ value /// leave @ 0 for the moment. may be irrelevant
### window - window - size

def window_list(data, window):
  if len(window) == 1:
    return(np.vstack([data.shift(i + 1) for i in range(window[0])][::-1]).T)
  else:
    return(np.vstack([data.shift(i + 1) for i in range(window[0], window[1])][::-1]).T)


## Current - Window list - (data, window - int)
## Ideal - window list - (data, [int] / [int, int])

# Feature_list -> (dict_object) window() -> window_list
# Feature_list -> (dict_object) window_stat() -> window_list
# Feature_list -> (dict_object) difference_comb() -> window_list



## Difference/Momentum/Force helper functions
def difference(data, lag):
  return(data.diff(lag))

def momentum(data, lag):
  return(difference(difference(data, lag), lag))

def force(data, lag):
  return(difference(momentum(data, lag), lag))



## Perform diff/mom/force operations
def difference_comb(data, window_lag):
  ## Check size of window_size
  if len(window_lag) == 2:
    ## Number of features to return
    cut = window_lag[0] - window_lag[1]
  else:
    cut = window_lag[1] - window_lag[2]

  ## Window size > Lag
  if cut <= 0:
    raise Exception("Window size must be greater than differencing")

  if len(window_lag) == 2:
    ## Difference Matrix
    multi_diff = difference(pd.DataFrame(window_list(data, [window_lag[0]-1])), window_lag[1])
    single_diff = difference(data, window_lag[1])

    multi_size = multi_diff.shape[1]
    multi_diff = multi_diff.iloc[:, window_lag[1]:]
  else:
    ## Difference Matrix
    multi_diff = difference(pd.DataFrame(window_list(data,[window_lag[0]-1 , window_lag[1]-1])), window_lag[2])
    single_diff = difference(data, window_lag[2])

    multi_size = multi_diff.shape[1]
    multi_diff = multi_diff.iloc[:, window_lag[2]:]

  ## Matrix result from operation
  diffed = np.column_stack((multi_diff, single_diff.rename(multi_size)))

  return(diffed)

  #return(pd.DataFrame(diffed))

def momentum_comb(data, window_lag):

  ## Check size of window_size
  if len(window_lag) == 2:
    ## Number of features to return
    cut = window_lag[0] - window_lag[1]
  else:
    cut = window_lag[1] - window_lag[2]


  if cut <= 0:
    raise Exception("Window size must be greater than differencing")

  if len(window_lag) == 2:
    ## Momentum Matrix
    multi_diff = momentum(pd.DataFrame(window_list(data, [window_lag[0]-1])), window_lag[1])
    single_diff = momentum(data, window_lag[1])

    multi_size = multi_diff.shape[1]
    multi_diff = multi_diff.iloc[:, window_lag[1]:]
  else:
    ## Momentum Matrix
    multi_diff = momentum(pd.DataFrame(window_list(data,[window_lag[0]-1 , window_lag[1]-1])), window_lag[2])
    single_diff = momentum(data, window_lag[2])

    multi_size = multi_diff.shape[1]
    multi_diff = multi_diff.iloc[:, window_lag[2]:]

  ## Matrix result from operation
  diffed = np.column_stack((multi_diff, single_diff.rename(multi_size)))

  return(diffed)

def force_comb(data, window_lag):

  ## Check size of window_size
  if len(window_lag) == 2:
  ## Number of features to return
    cut = window_lag[0] - window_lag[1]
  else:
    cut = window_lag[1] - window_lag[2]

  if cut <= 0:
    raise Exception("Window size must be greater than differencing")

  if len(window_lag) == 2:
    ## Force Matrix
    multi_diff = difference(pd.DataFrame(window_list(data, [window_lag[0]-1])), window_lag[1])
    single_diff = difference(data, window_lag[1])

    multi_size = multi_diff.shape[1]
    multi_diff = multi_diff.iloc[:, window_lag[1]:]
  else:
    ## Force Matrix
    multi_diff = difference(pd.DataFrame(window_list(data,[window_lag[0]-1 , window_lag[1]-1])), window_lag[2])
    single_diff = difference(data, window_lag[2])

    multi_size = multi_diff.shape[1]
    multi_diff = multi_diff.iloc[:, window_lag[2]:]

  ## Matrix result from operation
  diffed = np.column_stack((multi_diff, single_diff.rename(multi_size)))

  return(diffed)

### Get number of NaN values to drop
### returns NaNs at start & bottom of last T+ column.

def get_num_nan(df):
  num_nan = df.isnull().sum()
  return([max(num_nan[0:-1]), list(num_nan)[-1]])

## 2-d Matrix to Pandas Dataframe
## adds global index as DF index
## adds convenience for combination later
def build_data(data, index):
  data_df = pd.DataFrame(data)
  data_df["Date"] = index
  data_df = data_df.set_index("Date")
  return(data_df)

## Re-Centers data after multiple shifts
## uses num_nan output
def cut_final(df):
  h, t = get_num_nan(df)

  return(df[h:][:-t])



#https://www.researchgate.net/figure/Spectral-Statistical-and-Temporal-Domain-Features-used-in-the-present-work-1-Features_fig6_301683665
#https://gist.github.com/endolith/359724/aa7fcc043776f16f126a0ccd12b599499509c3cc

import statsmodels.api as sm
import numpy as np

axis = 1

def spectral_centroid(data):
    """Calculates spectral centroid

    Parameters
    ----------

    data : Series or vector
        Time series data to perform operation on

    Returns
    -------
    Array
        1-dim array

    """
    def _spectral(row):
        spectrum = abs(rfft(row))
        normalized_spectrum = spectrum / sum(spectrum)
        normalized_frequencies = linspace(0, 1, len(spectrum))
        return(sum(normalized_frequencies * normalized_spectrum))

    return(np.apply_along_axis(_spectral, data))



## Return cross correlation for Time Lag
import pandas as pd
import numpy as np
import scipy.stats

import seaborn as sns
import matplotlib.pyplot as plt
sns.set()



def get_lag_corr(y_actual, y_pred, num_lags):
    """Calculates & plots Lag Correlation

    Parameters
    ----------
    y_actual : Series or vector
        The ground truth
    y_pred : Series or vector
        Predicted values
    num_lags : int
        Lag to consider - range (0, num_lags)
    """
    lags = []
    for c in range(num_lags):
        lagged = pd.Series(y_pred).shift(c)
        lags.append(scipy.stats.spearmanr(lagged, y_actual, nan_policy='omit')[0])

    ax = sns.lineplot(x=range(len(lags)), y=lags, linestyle="-")
    ax.set(xlabel='Lag', ylabel='Corr Coefficient')


def actualPred(y_true, y_pred):
    """Plot actual vs predicted line plots

    Parameters
    ----------
    y_true : Series or vector
        The ground truth
    y_pred : Series or vector
        Predicted values
    """
    ax = sns.lineplot(x=range(y_true.shape[0]), y=y_true,
                      color="blue", label="Actual", linestyle="-")

    ax = sns.lineplot(x=range(y_pred.shape[0]), y=y_pred,
                  color="yellow", label="Predicted", linestyle="-")

    ax.set(xlabel='Time', ylabel='Y')


def scatter(y_true, y_pred):
    """Plot actual vs predicted scatterplot

    Parameters
    ----------
    y_true : Series or vector
        The ground truth
    y_pred : Series or vector
        Predicted values
    """
    ax = sns.scatterplot(x=y_true, y=y_pred)
    ax.set(xlabel='Actual', ylabel='Predicted')



from setuptools import find_packages, setup
import setuptools

with open("README.md", "r") as f:
    LONG_DESC = f.read()

setup(name="tsextract", 
      version='0.0.7',
      license='GNU GPL',
      url="https://github.com/cydal/tsExtract/tree/master/tsextract",
      description="Time series data preprocessing", 
      long_description=LONG_DESC,
      long_description_content_type="text/markdown",
      packages=find_packages(include=["tsextract", "tsextract.*"]),
      author="Sijuade Oguntayo", 
      author_email="cydalsij@outlook.com", 
      python_requires=">=3.6",
      install_requires=[
      "pandas >= 1.0.3",
      "seaborn >= 0.10.1",
      "statsmodels >= 0.10.2",
      "scipy >= 1.4.0",
      "matplotlib >= 3.2.1",
      "numpy >= 1.16.4"], 
      classifiers=[
         "Programming Language :: Python :: 3", 
         "Programming Language :: Python :: 3.6", 
         "Programming Language :: Python :: 3.7", 
         "License :: OSI Approved :: GNU General Public License (GPL)", 
         "Operating System :: OS Independent"
      ], 
      zip_safe=False)